# frozen_string_literal: true

require 'graphlient'
require 'ciflows/constants'
require 'byebug'

module Ciflows
  class Graphql
    attr_reader :workflow_id, :client, :schema, :job_name
    attr_accessor :workflow, :job, :failed_jobs

    def initialize(workflow_id, job_name)
      @workflow_id = workflow_id
      @job_name = job_name
      @client = Graphlient::Client.new(BASE_URL,
                                       headers: {
                                         'Authorization' => Ciflows::CI_TOKEN,
                                       })
      @schema = @client.schema
    end

    def approve_job
      client.query(id: job.id) do
        mutation(id: :ID!) do
          approveJob(id: :id)
        end
      end
    end

    def get_workflow
      @workflow ||= client.query(id: workflow_id) do
        query(id: :ID!) do
          workflow(id: :id) do
            status
            jobs do
              id
              name
              type
              status
            end
          end
        end
      end
    end

    def rerun_workflow_from(from = 'failed')
      client.query(id: workflow_id, from: from.upcase) do
        mutation(id: :ID!, from: :RerunWorkflowFromOption) do
          rerunWorkflow(id: :id, from: :from)
        end
      end
    end

    def get_jobs(status)
      @failed_jobs ||= workflow.data.workflow.jobs.select { |job| job.status.eql?(status) }
    end

    def job
      @job ||= workflow.data.workflow.jobs.detect { |job| job.type.eql?('APPROVAL') }
    end
  end
end
