# frozen_string_literal: true

require 'graphql/client'
require 'graphql/client/http'
require 'ciflows/constants'

module Ciflows
  class GraphqlV2
    HTTP = GraphQL::Client::HTTP.new('https://api.circleci.com/graphql-unstable') do
      def headers(_context)
        { "User-Agent": 'My Client', 'Authorization': Ciflows::CI_TOKEN }
      end
    end

    Schema = GraphQL::Client.load_schema(HTTP)
    Client = GraphQL::Client.new(schema: Schema, execute: HTTP)

    attr_reader :workflow_id, :client, :schema, :job_name
    attr_accessor :workflow, :job, :approve_job

    def initialize(workflow_id, job_name)
      @workflow_id = workflow_id
      @job_name = job_name
    end

    def approve_job
      @approve_job ||= Client.query(Mutation::ApproveJobMutation, variables: { id: job.id })
    end

    def get_workflow
      @workflow ||= Client.query(Query::WorkflowQuery, variables: { id: workflow_id })
    end

    def job
      @job ||= workflow.data.workflow.jobs.detect { |job| job.type.eql?("APPROVAL") }
    end
  end

  class Query
    WorkflowQuery = GraphqlV2::Client.parse <<~'GRAPHQL'
      query ($id: ID!) {
        workflow(id: $id) {
          status
          jobs {
            id
            name
            type
            status
          }
        }
      }
    GRAPHQL
  end

  class Mutation
    ApproveJobMutation = GraphqlV2::Client.parse <<~'GRAPHQL'
      mutation($id: ID!) {
        approveJob(id: $id)
      }
    GRAPHQL
  end
end
