# frozen_string_literal: true

require 'rest-client'
require 'json'
require 'ciflows/graphql'
require 'ciflows/graphql_v2'
require 'ciflows/constants'
require 'awesome_print'

module Ciflows
  class Project
    BASE_URL = 'https://circleci.com/api/v1/projects'

    attr_reader :repo_name, :branch_name, :build_name, :job_name
    attr_accessor :projects, :repo, :branch, :latest_workflows, :build

    def initialize(repo_name, branch_name, build_name, job_name)
      @branch_name = CGI.escape(branch_name)
      @repo_name = repo_name
      @build_name = build_name
      @job_name = job_name
    end

    def process
      find_projects
      find_repo
      find_branch
      find_latest_workflows
      find_build
    end

    def get_workflow
      graphql_client.get_workflow
    end

    def approve_workflow
      graphql_client.get_workflow
      graphql_client.approve_job
    end

    def rerun_workflow_from(from)
      graphql_client.get_workflow
      graphql_client.rerun_workflow_from(from)
    end

    def failed_jobs
      graphql_client.get_workflow
      graphql_client.get_jobs('FAILED')
    end

    private

    def graphql_client
      @graphql_client ||= Graphql.new(build['id'], job_name)
    end

    def find_projects
      res = RestClient.get(BASE_URL,
                           'Circle-Token' => Ciflows::CI_TOKEN,
                           content_type: :json,
                           accept: :json)

      @projects = JSON.parse(res.body)
    end

    def find_repo
      @repo ||= projects.detect { |proj| proj['reponame'] == repo_name }
    end

    def find_branch
      @branch ||= @repo['branches'][branch_name]
    end

    def find_latest_workflows
      @latest_workflows ||= branch['latest_workflows']
    end

    def find_build
      @build ||= begin
        if build_name
          latest_workflows[build_name]
        else
          build_name = latest_workflows.max_by { |_, data| data[:created_at] }[0]
          latest_workflows[build_name]
        end
      end
    end
  end
end
